package cn.iocoder.yudao.module.system.service.worklog;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.system.controller.admin.dept.vo.dept.DeptRespVO;
import cn.iocoder.yudao.module.system.controller.admin.worklog.vo.loginstance.LogInstancePageReqVO;
import cn.iocoder.yudao.module.system.controller.admin.worklog.vo.loginstance.LogInstanceRespVO;
import cn.iocoder.yudao.module.system.controller.admin.worklog.vo.loginstance.LogInstanceSaveReqVO;
import cn.iocoder.yudao.module.system.dal.dataobject.worklog.LogInstanceDO;
import com.baomidou.mybatisplus.core.metadata.IPage;

import javax.validation.Valid;
import java.util.List;
import java.util.Date;

/**
 * 日志实例的拓展 Service 接口
 *
 * @author 符溶馨
 */
public interface LogInstanceService {

    /**
     * 创建日志实例
     *
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createLogInstance(@Valid LogInstanceSaveReqVO createReqVO);

    /**
     * 更新日志实例
     *
     * @param updateReqVO 更新信息
     */
    void updateLogInstance(@Valid LogInstanceSaveReqVO updateReqVO);

    /**
     * 删除日志实例
     *
     * @param id 编号
     */
    void deleteLogInstance(Long id);

    /**
     * 获得日志实例
     *
     * @param id 编号
     * @return 日志实例的拓展
     */
    LogInstanceRespVO getLogInstance(Long id);

    /**
     * 获得日志实例分页
     *
     * @param pageReqVO 分页查询
     * @return 日志实例的拓展分页
     */
    IPage<LogInstanceRespVO> getLogInstancePage(LogInstancePageReqVO pageReqVO, Integer pagingType);

    /**
     * 获得我收到的日志页面中的部门列表
     */
    List<DeptRespVO> getDeptInfo();

    /**
     * 获取我管理的日志列表(时间区间)
     *
     * @return
     */
    List<LogInstanceDO> myManageLogInstanceByTimeRange(Long formId, Integer type, Date thisDate);

    /**
     * 获取我管理的日志列表(时间区间)
     *
     * @param fromId
     * @param beginTime
     * @param endTime
     * @return
     */
    List<LogInstanceDO> myManageLogInstanceByTimeRange(Long fromId, Date beginTime, Date endTime);

    List<LogInstanceDO> getNeedWriteLogInstanceByTimeRange(Long fromId, List<String> time, Long userId);

//    /**
//     * 获取我需要填写的日志
//     *
//     * @param fromId
//     * @param userId
//     * @return
//     */
//    List<LogInstanceDO> getNeedWriteLogInstanceByTimeRange(Long fromId, List<String> time, Long userId);
    LogInstanceDO getUpLog(Long formId, Integer type);

    LogInstanceRespVO setContentFun(LogInstanceRespVO logInstanceRespVO);
}